require 'hipchat'

module Pakyow
  module Fail
    module Handlers
      class Hipchat
        NICENAME = :hipchat

        # register and enable the handler
        Pakyow::Fail::Handler.register(NICENAME, self, %i[hipchat_room hipchat_api_token])
        Pakyow::Fail::Handler.enable(NICENAME)

        # define default config options
        Pakyow::Config.fail.opt(:hipchat_sender, lambda { "#{Pakyow::Config.app.name} Fail" })
        Pakyow::Config.fail.opt(:hipchat_color, :red)

        def initialize(app)
          req = app.req

          body = []
          body << "<strong>#{req.method.upcase} #{req.path} (#{req.error.class.name}) #{CGI.escapeHTML(req.error.message)}</strong>"
          body << "Params: #{app.params}"
          body << "Session: #{app.session}"
          body.concat(req.error.backtrace)

          client[Config.fail.hipchat_room].send(Config.fail.hipchat_sender, body.join('<br>'), color: Config.fail.hipchat_color)
        rescue StandardError => e
          app.logger.error "pakyow-hipchat failed: #{e.message}"
        end

        private

        def client
          @client ||= HipChat::Client.new(Config.fail.hipchat_api_token)
        end
      end
    end
  end
end
